# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from sgp_agents import Agents, AsyncAgents
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestHealthz:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: Agents) -> None:
        healthz = client.healthz.retrieve()
        assert_matches_type(object, healthz, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Agents) -> None:
        response = client.healthz.with_raw_response.retrieve()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        healthz = response.parse()
        assert_matches_type(object, healthz, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Agents) -> None:
        with client.healthz.with_streaming_response.retrieve() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            healthz = response.parse()
            assert_matches_type(object, healthz, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncHealthz:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncAgents) -> None:
        healthz = await async_client.healthz.retrieve()
        assert_matches_type(object, healthz, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncAgents) -> None:
        response = await async_client.healthz.with_raw_response.retrieve()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        healthz = await response.parse()
        assert_matches_type(object, healthz, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncAgents) -> None:
        async with async_client.healthz.with_streaming_response.retrieve() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            healthz = await response.parse()
            assert_matches_type(object, healthz, path=["response"])

        assert cast(Any, response.is_closed) is True
